// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
// This module is a shim for the plugin consumer in the browser
export function useIdentityPlugin(_plugin) {
    throw new Error("Identity plugins are not supported in browser environments.");
}//# sourceMappingURL=https://main.vscode-cdn.net/sourcemaps/6a44c352bd24569c417e530095901b649960f9f8/node_modules/@azure/identity/dist/browser/plugins/consumer-browser.mjs.map