// Copyright (c) Microsoft Corporation.
// Licensed under the MIT License.
export {};//# sourceMappingURL=https://main.vscode-cdn.net/sourcemaps/6a44c352bd24569c417e530095901b649960f9f8/node_modules/@azure/identity/dist/esm/credentials/visualStudioCodeCredentialOptions.js.map