/*! @azure/msal-browser v5.12.0 2026-06-05 */
'use strict';
/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
const SIGNIN_INITIATE = "/oauth2/v2.0/initiate";
const SIGNIN_CHALLENGE = "/oauth2/v2.0/challenge";
const SIGNIN_TOKEN = "/oauth2/v2.0/token";
const SIGNIN_INTROSPECT = "/oauth2/v2.0/introspect";
const SIGNUP_START = "/signup/v1.0/start";
const SIGNUP_CHALLENGE = "/signup/v1.0/challenge";
const SIGNUP_CONTINUE = "/signup/v1.0/continue";
const RESET_PWD_START = "/resetpassword/v1.0/start";
const RESET_PWD_CHALLENGE = "/resetpassword/v1.0/challenge";
const RESET_PWD_CONTINUE = "/resetpassword/v1.0/continue";
const RESET_PWD_SUBMIT = "/resetpassword/v1.0/submit";
const RESET_PWD_POLL = "/resetpassword/v1.0/poll_completion";
const REGISTER_INTROSPECT = "/register/v1.0/introspect";
const REGISTER_CHALLENGE = "/register/v1.0/challenge";
const REGISTER_CONTINUE = "/register/v1.0/continue";

export { REGISTER_CHALLENGE, REGISTER_CONTINUE, REGISTER_INTROSPECT, RESET_PWD_CHALLENGE, RESET_PWD_CONTINUE, RESET_PWD_POLL, RESET_PWD_START, RESET_PWD_SUBMIT, SIGNIN_CHALLENGE, SIGNIN_INITIATE, SIGNIN_INTROSPECT, SIGNIN_TOKEN, SIGNUP_CHALLENGE, SIGNUP_CONTINUE, SIGNUP_START };
//# sourceMappingURL=CustomAuthApiEndpoint.mjs.map
