"use strict";
/*
 * Copyright The OpenTelemetry Authors
 * SPDX-License-Identifier: Apache-2.0
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getInstrumentationScopeKey = void 0;
/**
 * Converting the instrumentation scope object to a unique identifier string.
 * @param scope - The instrumentation scope to convert
 * @returns A unique string identifier for the scope
 */
function getInstrumentationScopeKey(scope) {
    return `${scope.name}@${scope.version || ''}:${scope.schemaUrl || ''}`;
}
exports.getInstrumentationScopeKey = getInstrumentationScopeKey;
//# sourceMappingURL=utils.js.map