/*
 * Copyright The OpenTelemetry Authors
 * SPDX-License-Identifier: Apache-2.0
 */
export class NoopDetector {
    detect() {
        return {
            attributes: {},
        };
    }
}
export const noopDetector = new NoopDetector();
//# sourceMappingURL=NoopDetector.js.map