export { AzureKeyCredential } from "./azureKeyCredential.js";
export { isKeyCredential } from "./keyCredential.js";
export { AzureNamedKeyCredential, isNamedKeyCredential, } from "./azureNamedKeyCredential.js";
export { AzureSASCredential, isSASCredential } from "./azureSASCredential.js";
export { isTokenCredential, } from "./tokenCredential.js";//# sourceMappingURL=https://main.vscode-cdn.net/sourcemaps/6a44c352bd24569c417e530095901b649960f9f8/node_modules/@azure/core-auth/dist/browser/index.js.map