/*! @azure/msal-browser v5.12.0 2026-06-05 */
'use strict';
/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
const SIGN_IN_CODE_SEND_RESULT_TYPE = "SignInCodeSendResult";
const SIGN_IN_PASSWORD_REQUIRED_RESULT_TYPE = "SignInPasswordRequiredResult";
const SIGN_IN_COMPLETED_RESULT_TYPE = "SignInCompletedResult";
const SIGN_IN_JIT_REQUIRED_RESULT_TYPE = "SignInJitRequiredResult";
const SIGN_IN_MFA_REQUIRED_RESULT_TYPE = "SignInMfaRequiredResult";
function createSignInCompleteResult(input) {
    return {
        type: SIGN_IN_COMPLETED_RESULT_TYPE,
        ...input,
    };
}
function createSignInPasswordRequiredResult(input) {
    return {
        type: SIGN_IN_PASSWORD_REQUIRED_RESULT_TYPE,
        ...input,
    };
}
function createSignInCodeSendResult(input) {
    return {
        type: SIGN_IN_CODE_SEND_RESULT_TYPE,
        ...input,
    };
}
function createSignInJitRequiredResult(input) {
    return {
        type: SIGN_IN_JIT_REQUIRED_RESULT_TYPE,
        ...input,
    };
}
function createSignInMfaRequiredResult(input) {
    return {
        type: SIGN_IN_MFA_REQUIRED_RESULT_TYPE,
        ...input,
    };
}

export { SIGN_IN_CODE_SEND_RESULT_TYPE, SIGN_IN_COMPLETED_RESULT_TYPE, SIGN_IN_JIT_REQUIRED_RESULT_TYPE, SIGN_IN_MFA_REQUIRED_RESULT_TYPE, SIGN_IN_PASSWORD_REQUIRED_RESULT_TYPE, createSignInCodeSendResult, createSignInCompleteResult, createSignInJitRequiredResult, createSignInMfaRequiredResult, createSignInPasswordRequiredResult };
//# sourceMappingURL=SignInActionResult.mjs.map
