"use strict";
/*---------------------------------------------------------------------------------------------
 *  Copyright (c) Microsoft Corporation. All rights reserved.
 *  Licensed under the MIT License. See License.txt in the project root for license information.
 *--------------------------------------------------------------------------------------------*/
Object.defineProperty(exports, "__esModule", { value: true });
exports.ExperimentationService = void 0;
var ExperimentationService_1 = require("./tas-client/ExperimentationService");
Object.defineProperty(exports, "ExperimentationService", { enumerable: true, get: function () { return ExperimentationService_1.ExperimentationService; } });
//# sourceMappingURL=index.js.map