/*! @azure/msal-browser v5.12.0 2026-06-05 */
'use strict';
/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
const NoNetworkConnectivity = "no_network_connectivity";
const FailedSendRequest = "failed_send_request";

export { FailedSendRequest, NoNetworkConnectivity };
//# sourceMappingURL=HttpErrorCodes.mjs.map
