"use strict";
// Copyright (c) Microsoft Corporation.
// Licensed under the MIT license.
Object.defineProperty(exports, "__esModule", { value: true });
exports.tunnelSdkUserAgent = void 0;
// Get the package version
const packageJson = require("./package.json");
const packageVersion = packageJson.version;
/**
 * Tunnel SDK user agent
 */
exports.tunnelSdkUserAgent = `Dev-Tunnels-Service-TypeScript-SDK/${packageVersion}`;
//# sourceMappingURL=https://main.vscode-cdn.net/sourcemaps/6a44c352bd24569c417e530095901b649960f9f8/node_modules/@microsoft/dev-tunnels-management/version.js.map