/*! @azure/msal-browser v5.12.0 2026-06-05 */
'use strict';
import { AuthFlowStateBase } from '../../AuthFlowState.mjs';
import { MFA_FAILED_STATE_TYPE } from '../../AuthFlowStateTypes.mjs';

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
/**
 * State indicating that the MFA flow has failed.
 */
class MfaFailedState extends AuthFlowStateBase {
    constructor() {
        super(...arguments);
        /**
         * The type of the state.
         */
        this.stateType = MFA_FAILED_STATE_TYPE;
    }
}

export { MfaFailedState };
//# sourceMappingURL=MfaFailedState.mjs.map
