"use strict";
/*
 * Copyright The OpenTelemetry Authors
 * SPDX-License-Identifier: Apache-2.0
 */
Object.defineProperty(exports, "__esModule", { value: true });
exports.getStringListFromEnv = exports.getNumberFromEnv = exports.getBooleanFromEnv = exports.getStringFromEnv = void 0;
function getStringFromEnv(_) {
    return undefined;
}
exports.getStringFromEnv = getStringFromEnv;
function getBooleanFromEnv(_) {
    return undefined;
}
exports.getBooleanFromEnv = getBooleanFromEnv;
function getNumberFromEnv(_) {
    return undefined;
}
exports.getNumberFromEnv = getNumberFromEnv;
function getStringListFromEnv(_) {
    return undefined;
}
exports.getStringListFromEnv = getStringListFromEnv;
//# sourceMappingURL=environment.js.map