/*! @azure/msal-browser v5.12.0 2026-06-05 */
'use strict';
import { CustomAuthError } from './CustomAuthError.mjs';

/*
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License.
 */
class InvalidConfigurationError extends CustomAuthError {
    constructor(error, message, correlationId) {
        super(error, message, correlationId);
        Object.setPrototypeOf(this, InvalidConfigurationError.prototype);
    }
}

export { InvalidConfigurationError };
//# sourceMappingURL=InvalidConfigurationError.mjs.map
