"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.DiagnosticMessageId = void 0;
exports.DiagnosticMessageId = {
    "attachSuccessful": "3000",
    "sdkExists": "3001",
    "missingIkey": "3002",
    "setupAlreadyCalled": "3003",
    "prefixFailed": "3004",
    "aadEnabled": "3005",
    "unknownError": "3006"
};
//# sourceMappingURL=DataModel.js.map