"use strict";

Object.defineProperty(exports, "__esModule", {
  value: true
});
Object.defineProperty(exports, "asConst", {
  enumerable: true,
  get: function get() {
    return _utils.asConst;
  }
});
Object.defineProperty(exports, "wrapCompilerAsTypeGuard", {
  enumerable: true,
  get: function get() {
    return _utils.wrapCompilerAsTypeGuard;
  }
});
Object.defineProperty(exports, "wrapValidatorAsTypeGuard", {
  enumerable: true,
  get: function get() {
    return _utils.wrapValidatorAsTypeGuard;
  }
});

var _utils = require("./utils");
//# sourceMappingURL=index.js.map