"use strict";
// Original file: null
Object.defineProperty(exports, "__esModule", { value: true });
exports._google_protobuf_GeneratedCodeInfo_Annotation_Semantic = void 0;
// Original file: null
exports._google_protobuf_GeneratedCodeInfo_Annotation_Semantic = {
    NONE: 'NONE',
    SET: 'SET',
    ALIAS: 'ALIAS',
};
//# sourceMappingURL=GeneratedCodeInfo.js.map